@@ -16,16 +16,106 @@
 #include <QImage>
 #include <QImageReader>
 #include <QImageWriter>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QJsonObject>
+#include <QJsonParseError>
 #include <QMetaEnum>
 #include <QTextStream>
 
 #include "fuzzyeq.cpp"
 
+QJsonObject readOptionalInfo(const QString &suffix)
+{
+    auto fi = QFileInfo(QStringLiteral("%1/basic/%2.json").arg(IMAGEDIR, suffix));
+    if (!fi.exists()) {
+        return {};
+    }
+
+    QFile f(fi.filePath());
+    if (!f.open(QFile::ReadOnly)) {
+        return {};
+    }
+
+    QJsonParseError err;
+    auto doc = QJsonDocument::fromJson(f.readAll(), &err);
+    if (err.error != QJsonParseError::NoError || !doc.isObject()) {
+        return {};
+    }
+
+    return doc.object();
+}
+
+void setOptionalInfo(QImage& image, const QString &suffix)
+{
+    auto obj = readOptionalInfo(suffix);
+    if (obj.isEmpty()) {
+        return ;
+    }
+
+    // Set resolution
+    auto res = obj.value("resolution").toObject();
+    if (!res.isEmpty()) {
+        image.setDotsPerMeterX(res.value("dotsPerMeterX").toInt());
+        image.setDotsPerMeterY(res.value("dotsPerMeterY").toInt());
+    }
+
+    // Set metadata
+    auto meta = obj.value("metadata").toArray();
+    for (auto jv : meta) {
+        auto obj = jv.toObject();
+        auto key = obj.value("key").toString();
+        auto val = obj.value("value").toString();
+        image.setText(key, val);
+    }
+}
+
+bool checkOptionalInfo(QImage& image, const QString &suffix)
+{
+    auto obj = readOptionalInfo(suffix);
+    if (obj.isEmpty()) {
+        return true;
+    }
+
+    // Check if the format match
+    if (suffix.compare(obj.value("format").toString(), Qt::CaseInsensitive)) {
+        return false;
+    }
+
+    // Test resolution
+    auto res = obj.value("resolution").toObject();
+    if (!res.isEmpty()) {
+        auto resx = res.value("dotsPerMeterX").toInt();
+        auto resy = res.value("dotsPerMeterY").toInt();
+        if (resx != image.dotsPerMeterX()) {
+            return false;
+        }
+        if (resy != image.dotsPerMeterY()) {
+            return false;
+        }
+    }
+
+    // Test metadata
+    auto meta = obj.value("metadata").toArray();
+    for (auto jv : meta) {
+        auto obj = jv.toObject();
+        auto key = obj.value("key").toString();
+        auto val = obj.value("value").toString();
+        auto cur = image.text(key);
+        if (cur != val) {
+            qDebug() << key;
+            return false;
+        }
+    }
+
+    return true;
+}
+
 /*!
  * \brief basicTest
  * Run a basic test on some common images.
  */
-int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, uint fuzzarg)
+int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, bool skipOptional, uint fuzzarg)
 {
     uchar fuzziness = uchar(fuzzarg);
 
@@ -43,7 +133,7 @@ int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, uint f
     int failed = 0;
 
     QTextStream(stdout) << "********* "
-                        << "Starting basic write tests for " << suffix << " images *********\n";
+                        << "Starting BASIC write tests for " << suffix << " images *********\n";
     const QFileInfoList lstImgDir = imgdir.entryInfoList();
     for (const QFileInfo &fi : lstImgDir) {
         QString pngfile;
@@ -70,6 +160,9 @@ int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, uint f
             if (lossless) {
                 imgWriter.setQuality(100);
             }
+            if (!skipOptional) {
+                setOptionalInfo(pngImage, suffix);
+            }
             if (!imgWriter.write(pngImage)) {
                 QTextStream(stdout) << "FAIL : " << fi.fileName() << ": failed to write image data\n";
                 ++failed;
@@ -96,7 +189,6 @@ int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, uint f
                     continue;
                 }
             }
-
             if (expData != writtenData) {
                 QTextStream(stdout) << "FAIL : " << fi.fileName() << ": written data differs from " << fi.fileName() << "\n";
                 ++failed;
@@ -113,6 +205,13 @@ int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, uint f
                 ++failed;
                 continue;
             }
+            if (!skipOptional) {
+                if (!checkOptionalInfo(reReadImage, suffix)) {
+                    QTextStream(stdout) << "FAIL : " << fi.fileName() << ": optional information does not match\n";
+                    ++failed;
+                    continue;
+                }
+            }
             if (reReadImage.colorSpace().isValid()) {
                 QColorSpace toColorSpace;
                 if (pngImage.colorSpace().isValid()) {
@@ -148,7 +247,7 @@ int basicTest(const QString &suffix, bool lossless, bool ignoreDataCheck, uint f
 
     QTextStream(stdout) << "Totals: " << passed << " passed, " << failed << " failed\n";
     QTextStream(stdout) << "********* "
-                        << "Finished basic write tests for " << suffix << " images *********\n";
+                        << "Finished BASIC write tests for " << suffix << " images *********\n";
 
     return failed == 0 ? 0 : 1;
 }
@@ -238,7 +337,7 @@ int formatTest(const QString &suffix, bool createTemplates)
     int skipped = 0;
 
     QTextStream(stdout) << "********* "
-                        << "Starting format write tests for " << suffix << " images *********\n";
+                        << "Starting FORMAT write tests for " << suffix << " images *********\n";
 
     for (int i = QImage::Format_Invalid + 1; i < QImage::NImageFormats; ++i) {
         auto format = QImage::Format(i);
@@ -304,7 +403,8 @@ int formatTest(const QString &suffix, bool createTemplates)
         // checking the format: must be the same
         if (writtenImage.format() != tmplImage.format()) {
             ++failed;
-            QTextStream(stdout) << "FAIL : format mismatch " << formatName << "\n";
+            auto tmplformatName = QString(QMetaEnum::fromType<QImage::Format>().valueToKey(tmplImage.format()));
+            QTextStream(stdout) << "FAIL : format mismatch " << formatName << " != " << tmplformatName << "\n";
             continue;
         }
 
@@ -322,6 +422,142 @@ int formatTest(const QString &suffix, bool createTemplates)
         ++passed;
     }
 
+    QTextStream(stdout) << "Totals: " << passed << " passed, " << failed << " failed, " << skipped << " skipped\n";
+    QTextStream(stdout) << "********* "
+                        << "Finished FORMAT write tests for " << suffix << " images *********\n";
+
+    return failed == 0 ? 0 : 1;
+}
+
+int sizeTest(const QString &suffix)
+{
+    auto formatFolder = QStringLiteral("%1/format/%2").arg(IMAGEDIR, suffix);
+
+    int passed = 0;
+    int failed = 0;
+    int skipped = 0;
+
+    QTextStream(stdout) << "********* "
+                        << "Starting SIZE write tests for " << suffix << " images *********\n";
+
+    for (int i = QImage::Format_Invalid + 1; i < QImage::NImageFormats; ++i) {
+        auto format = QImage::Format(i);
+        auto formatName = QString(QMetaEnum::fromType<QImage::Format>().valueToKey(format));
+
+        // read template image
+        auto templateImage = QImageReader(QStringLiteral("%1/%2.%3").arg(formatFolder, formatName, suffix)).read();
+        if (templateImage.isNull()) {
+            ++skipped;
+            QTextStream(stdout) << "SKIP : no source image found for " << formatName << "\n";
+            continue;
+        }
+
+        // clang-format off
+        auto sizeList = QList<QSize>()
+            // check for byte alignment
+            << (templateImage.size() + QSize(1, 1))
+            << (templateImage.size() + QSize(2, 2))
+            << (templateImage.size() + QSize(3, 3))
+            << (templateImage.size() + QSize(4, 4))
+            << (templateImage.size() + QSize(5, 5))
+            << (templateImage.size() + QSize(6, 6))
+            << (templateImage.size() + QSize(7, 7))
+            // Check for dividers (using prime numbers)
+            << QSize(113, 157)
+            << QSize(163, 109);
+        // clang-format on
+
+        for (auto &&sz : sizeList) {
+            auto debugInfo = QStringLiteral("%1 @%2x%3").arg(formatName).arg(sz.width()).arg(sz.height());
+
+            // resize the template image
+            auto resizeImage = templateImage.scaled(sz);
+            // make sure the format is not changed by resize (paranoia)
+            resizeImage.convertTo(templateImage.format());
+
+            // save test
+            QByteArray ba;
+            { // writing the image
+                QBuffer buffer(&ba);
+                QImageWriter writer(&buffer, suffix.toLatin1());
+                writer.setQuality(100); // always lossless
+                if (!writer.write(resizeImage)) {
+                    ++failed;
+                    QTextStream(stdout) << "FAIL : failed to write image " << debugInfo << "\n";
+                    continue;
+                }
+            }
+
+            // read test
+            QBuffer buffer(&ba);
+            auto writtenImage = QImageReader(&buffer, suffix.toLatin1()).read();
+            if (writtenImage.isNull()) {
+                ++failed;
+                QTextStream(stdout) << "FAIL : error while reading the image " << debugInfo << "\n";
+                continue;
+            }
+
+            // This comparison is only to understand if the plugin has written a completely wrong image. I therefore have no
+            // qualms about converting them to a more convenient format or to tolerate slightly different pixels.
+            auto compareFormat = writtenImage.hasAlphaChannel() ? QImage::Format_ARGB32 : QImage::Format_RGB32;
+            if (!fuzzyeq(writtenImage.convertToFormat(compareFormat), resizeImage.convertToFormat(compareFormat), 5)) {
+                ++failed;
+                QTextStream(stdout) << "FAIL : re-reading the data resulted in a different image " << debugInfo << "\n";
+                continue;
+            }
+
+            // test passed!
+            QTextStream(stdout) << "PASS : " << debugInfo << "\n";
+            ++passed;
+        }
+    }
+
+    QTextStream(stdout) << "Totals: " << passed << " passed, " << failed << " failed, " << skipped << " skipped\n";
+    QTextStream(stdout) << "********* "
+                        << "Finished SIZE write tests for " << suffix << " images *********\n";
+
+    return failed == 0 ? 0 : 1;
+}
+
+/*!
+ * \brief nullDeviceTest
+ * Checks the plugin behaviour when using a NULL device.
+ */
+int nullDeviceTest(const QString &suffix)
+{
+    QTextStream(stdout) << "********* "
+                        << "Starting NULL device write tests for " << suffix << " images *********\n";
+
+    int passed = 0;
+    int failed = 0;
+    int skipped = 0;
+
+    QImageWriter writer;
+    writer.setFormat(suffix.toLatin1());
+
+    if (writer.canWrite()) {
+        QTextStream(stdout) << "FAIL : canWrite() returns TRUE\n";
+        ++failed;
+    }
+
+    if (writer.write(QImage(16, 16, QImage::Format_ARGB32))) {
+        QTextStream(stdout) << "FAIL : write() returns TRUE\n";
+        ++failed;
+    }
+
+    // test for crash only
+    writer.compression();
+    writer.quality();
+    writer.transformation();
+    writer.subType();
+    writer.supportedSubTypes();
+    writer.optimizedWrite();
+    writer.progressiveScanWrite();
+
+    if (failed == 0) {// success
+        ++passed;
+    }
+
     QTextStream(stdout) << "Totals: " << passed << " passed, " << failed << " failed, " << skipped << " skipped\n";
     QTextStream(stdout) << "********* "
                         << "Finished format write tests for " << suffix << " images *********\n";
@@ -350,10 +586,14 @@ int main(int argc, char **argv)
                             QStringLiteral("max"));
     QCommandLineOption createFormatTempates({QStringLiteral("create-format-templates")},
                                             QStringLiteral("Create template images for all formats supported by QImage."));
+    QCommandLineOption skipOptTest({QStringLiteral("skip-optional-tests")},
+                                   QStringLiteral("Skip optional data tests (metadata, resolution, etc.)."));
+
     parser.addOption(lossless);
     parser.addOption(ignoreDataCheck);
     parser.addOption(fuzz);
     parser.addOption(createFormatTempates);
+    parser.addOption(skipOptTest);
 
     parser.process(app);
 
@@ -378,10 +618,16 @@ int main(int argc, char **argv)
 
     // run test
     auto suffix = args.at(0);
-    auto ret = basicTest(suffix, parser.isSet(lossless), parser.isSet(ignoreDataCheck), fuzzarg);
+    auto ret = basicTest(suffix, parser.isSet(lossless), parser.isSet(ignoreDataCheck), parser.isSet(skipOptTest), fuzzarg);
     if (ret == 0) {
         ret = formatTest(suffix, parser.isSet(createFormatTempates));
     }
+    if (ret == 0) {
+        ret = sizeTest(suffix);
+    }
+    if (ret == 0) {
+        ret = nullDeviceTest(suffix);
+    }
 
     return ret;
 }
